@@ -52,13 +52,13 @@
 #include "rd.h"
 #include "rdtime.h"
 
-#ifdef _MSC_VER
+#ifdef _WIN32
 #include "../win32/wingetopt.h"
 #include "../win32/wintime.h"
 #endif
 
 
-static int run = 1;
+static volatile sig_atomic_t run = 1;
 static int forever = 1;
 static rd_ts_t dispintvl = 1000;
 static int do_seq = 0;
@@ -774,7 +774,7 @@ static rd_kafka_resp_err_t do_produce (rd_kafka_t *rk,
  */
 static void do_sleep (int sleep_us) {
         if (sleep_us > 100) {
-#ifdef _MSC_VER
+#ifdef _WIN32
                 Sleep(sleep_us / 1000);
 #else
                 usleep(sleep_us);
@@ -863,7 +863,7 @@ int main (int argc, char **argv) {
 	while ((opt =
 		getopt(argc, argv,
 		       "PCG:t:p:b:s:k:c:fi:MDd:m:S:x:"
-                       "R:a:z:o:X:B:eT:Y:qvIur:lA:OwNHH:")) != -1) {
+                       "R:a:z:o:X:B:eT:Y:qvIur:lA:OwNH:")) != -1) {
 		switch (opt) {
 		case 'G':
 			if (rd_kafka_conf_set(conf, "group.id", optarg,
@@ -962,37 +962,32 @@ int main (int argc, char **argv) {
 		case 'd':
 			debug = optarg;
 			break;
-                case 'H':
-                {
-                        char *name, *val;
-                        size_t name_sz = -1;
+        case 'H':
+            if (!strcmp(optarg, "parse"))
+                read_hdrs = 1;
+            else {
+                char *name, *val;
+                size_t name_sz = -1;
 
-                        if (!optarg) {
-                                read_hdrs = 1;
-                                break;
-                        }
-
-                        name = optarg;
-                        val = strchr(name, '=');
-                        if (val) {
-                                name_sz = (size_t)(val-name);
-                                val++; /* past the '=' */
-                        }
-
-                        if (!hdrs)
-                                hdrs = rd_kafka_headers_new(8);
-
-                        err = rd_kafka_header_add(hdrs, name, name_sz, val, -1);
-                        if (err) {
-                                fprintf(stderr,
-                                        "%% Failed to add header %s: %s\n",
-                                        name, rd_kafka_err2str(err));
-                                exit(1);
-                        }
-
-                        read_hdrs = 1;
+                name = optarg;
+                val = strchr(name, '=');
+                if (val) {
+                        name_sz = (size_t)(val-name);
+                        val++; /* past the '=' */
                 }
-                break;
+
+                if (!hdrs)
+                        hdrs = rd_kafka_headers_new(8);
+
+                err = rd_kafka_header_add(hdrs, name, name_sz, val, -1);
+                if (err) {
+                        fprintf(stderr,
+                                "%% Failed to add header %s: %s\n",
+                                name, rd_kafka_err2str(err));
+                        exit(1);
+                }
+            }
+            break;
 		case 'X':
 		{
 			char *name, *val;
@@ -1131,8 +1126,8 @@ int main (int argc, char **argv) {
 			"  -b <brokers> Broker address list (host[:port],..)\n"
 			"  -s <size>    Message size (producer)\n"
 			"  -k <key>     Message key (producer)\n"
-                        "  -H <name[=value]> Add header to message (producer)\n"
-                        "  -H           Read message headers (consumer)\n"
+            "  -H <name[=value]> Add header to message (producer)\n"
+            "  -H parse     Read message headers (consumer)\n"
 			"  -c <cnt>     Messages to transmit/receive\n"
 			"  -x <cnt>     Hard exit after transmitting <cnt> messages (producer)\n"
 			"  -D           Copy/Duplicate data buffer (producer)\n"
@@ -1263,9 +1258,15 @@ int main (int argc, char **argv) {
         if (stats_intvlstr) {
                 /* User enabled stats (-T) */
 
-#ifndef _MSC_VER
+#ifndef _WIN32
                 if (stats_cmd) {
-                        if (!(stats_fp = popen(stats_cmd, "we"))) {
+                        if (!(stats_fp = popen(stats_cmd,
+#ifdef __linux__
+                                               "we"
+#else
+                                               "w"
+#endif
+                                               ))) {
                                 fprintf(stderr,
                                         "%% Failed to start stats command: "
                                         "%s: %s", stats_cmd, strerror(errno));
@@ -1287,6 +1288,15 @@ int main (int argc, char **argv) {
         if (mode == 'C' || mode == 'G')
                 rd_kafka_conf_set(conf, "enable.partition.eof", "true",
                                   NULL, 0);
+    if (read_hdrs && mode == 'P') {
+        fprintf(stderr, "%% producer can not read headers\n");
+        exit(1);
+    }
+    
+    if (hdrs && mode != 'P') {
+        fprintf(stderr, "%% consumer can not add headers\n");
+        exit(1);
+    }
 
 	if (mode == 'P') {
 		/*
@@ -1543,7 +1553,6 @@ int main (int argc, char **argv) {
 			fetch_latency = rd_clock();
 
 			if (batch_size) {
-				int i;
 				int partition = partitions ? partitions[0] :
 				    RD_KAFKA_PARTITION_UA;
 
@@ -1553,7 +1562,7 @@ int main (int argc, char **argv) {
 							   rkmessages,
 							   batch_size);
 				if (r != -1) {
-					for (i = 0 ; i < r ; i++) {
+					for (i = 0 ; (ssize_t)i < r ; i++) {
 						msg_consume(rkmessages[i],
 							NULL);
 						rd_kafka_message_destroy(
@@ -1611,7 +1620,6 @@ int main (int argc, char **argv) {
 		/*
 		 * High-level balanced Consumer
 		 */
-		rd_kafka_resp_err_t err;
 
 		rd_kafka_conf_set_rebalance_cb(conf, rebalance_cb);
 		rd_kafka_conf_set_default_topic_conf(conf, topic_conf);
@@ -1692,7 +1700,7 @@ int main (int argc, char **argv) {
 		fclose(latency_fp);
 
         if (stats_fp) {
-#ifndef _MSC_VER
+#ifndef _WIN32
                 pclose(stats_fp);
 #endif
                 stats_fp = NULL;
