@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 #
 # NuGet packaging script.
 # Assembles a NuGet package using CI artifacts in S3
@@ -18,6 +18,11 @@ from collections import defaultdict
 import boto3
 from zfile import zfile
 
+if sys.version_info[0] < 3:
+    from urllib import unquote
+else:
+    from urllib.parse import unquote
+
 
 # Rename token values
 rename_vals = {'plat': {'windows': 'win'},
@@ -42,11 +47,16 @@ rename_vals = {'plat': {'windows': 'win'},
 #  sha     - git sha
 #  bid     - builder's build-id
 #  bldtype - Release, Debug (appveyor)
+#  lnk     - std, static
 #
 # Example:
 #   librdkafka/p-librdkafka__bld-travis__plat-linux__arch-x64__tag-v0.0.62__sha-d051b2c19eb0c118991cd8bc5cf86d8e5e446cde__bid-1562.1/librdkafka.tar.gz
 
 
+class MissingArtifactError(Exception):
+    pass
+
+
 s3_bucket = 'librdkafka-ci-packages'
 dry_run = False
 
@@ -330,7 +340,7 @@ class NugetPackage (Package):
             # Travis Ubuntu 14.04 build
             [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'librdkafka-gcc.tar.gz'}, './lib/librdkafka.so.1', 'runtimes/linux-x64/native/librdkafka.so'],
             # Travis CentOS 7 RPM build
-            [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'librdkafka1*.x86_64.rpm'}, './usr/lib64/librdkafka.so.1', 'runtimes/linux-x64/native/centos7-librdkafka.so'],
+            [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'librdkafka1*el7.x86_64.rpm'}, './usr/lib64/librdkafka.so.1', 'runtimes/linux-x64/native/centos7-librdkafka.so'],
             # Alpine build
             [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'alpine-librdkafka.tgz'}, 'librdkafka.so.1', 'runtimes/linux-x64/native/alpine-librdkafka.so'],
 
@@ -394,7 +404,7 @@ class NugetPackage (Package):
                     break
 
             if artifact is None:
-                raise Exception('unable to find artifact with tags %s matching "%s"' % (str(attributes), fname_glob))
+                raise MissingArtifactError('unable to find artifact with tags %s matching "%s"' % (str(attributes), fname_glob))
 
             outf = os.path.join(self.stpath, m[2])
             member = m[1]
@@ -451,7 +461,126 @@ class NugetPackage (Package):
             print('Verifying %s:' % path)
 
             # Zipfiles may url-encode filenames, unquote them before matching.
-            pkgd = [urllib.unquote(x) for x in zf.getnames()]
+            pkgd = [unquote(x) for x in zf.getnames()]
+            missing = [x for x in expect if x not in pkgd]
+
+        if len(missing) > 0:
+            print('Missing files in package %s:\n%s' % (path, '\n'.join(missing)))
+            return False
+        else:
+            print('OK - %d expected files found' % len(expect))
+            return True
+
+
+class StaticPackage (Package):
+    """ Create a package with all static libraries """
+
+    # Only match statically linked artifacts
+    match = {'lnk': 'static'}
+
+    def __init__ (self, version, arts):
+        super(StaticPackage, self).__init__(version, arts, "static")
+
+    def cleanup(self):
+        if os.path.isdir(self.stpath):
+            shutil.rmtree(self.stpath)
+
+    def build (self, buildtype):
+        """ Build single package for all artifacts. """
+
+        self.stpath = tempfile.mkdtemp(prefix="out-", dir=".")
+
+        mappings = [
+            # rdkafka.h
+            [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'librdkafka-clang.tar.gz'}, './include/librdkafka/rdkafka.h', 'rdkafka.h'],
+
+            # LICENSES.txt
+            [{'arch': 'x64', 'plat': 'osx', 'fname_glob': 'librdkafka-clang.tar.gz'}, './share/doc/librdkafka/LICENSES.txt', 'LICENSES.txt'],
+
+            # glibc linux static lib and pkg-config file
+            [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'librdkafka-clang.tar.gz'}, './lib/librdkafka-static.a', 'librdkafka_glibc_linux.a'],
+            [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'librdkafka-clang.tar.gz'}, './lib/pkgconfig/rdkafka-static.pc', 'librdkafka_glibc_linux.pc'],
+
+            # musl linux static lib and pkg-config file
+            [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'alpine-librdkafka.tgz'}, 'librdkafka-static.a', 'librdkafka_musl_linux.a'],
+            [{'arch': 'x64', 'plat': 'linux', 'fname_glob': 'alpine-librdkafka.tgz'}, 'rdkafka-static.pc', 'librdkafka_musl_linux.pc'],
+
+            # osx static lib and pkg-config file
+            [{'arch': 'x64', 'plat': 'osx', 'fname_glob': 'librdkafka-clang.tar.gz'}, './lib/librdkafka-static.a', 'librdkafka_darwin.a'],
+            [{'arch': 'x64', 'plat': 'osx', 'fname_glob': 'librdkafka-clang.tar.gz'}, './lib/pkgconfig/rdkafka-static.pc', 'librdkafka_darwin.pc'],
+        ]
+
+        for m in mappings:
+            attributes = m[0].copy()
+            attributes.update(self.match)
+            fname_glob = attributes['fname_glob']
+            del attributes['fname_glob']
+            fname_excludes = []
+            if 'fname_excludes' in attributes:
+                fname_excludes = attributes['fname_excludes']
+                del attributes['fname_excludes']
+
+            artifact = None
+            for a in self.arts.artifacts:
+                found = True
+
+                for attr in attributes:
+                    if attr not in a.info or a.info[attr] != attributes[attr]:
+                        found = False
+                        break
+
+                if not fnmatch(a.fname, fname_glob):
+                    found = False
+
+                for exclude in fname_excludes:
+                    if exclude in a.fname:
+                        found = False
+                        break
+
+                if found:
+                    artifact = a
+                    break
+
+            if artifact is None:
+                raise MissingArtifactError('unable to find artifact with tags %s matching "%s"' % (str(attributes), fname_glob))
+
+            outf = os.path.join(self.stpath, m[2])
+            member = m[1]
+            try:
+                zfile.ZFile.extract(artifact.lpath, member, outf)
+            except KeyError as e:
+                raise Exception('file not found in archive %s: %s. Files in archive are: %s' % (artifact.lpath, e, zfile.ZFile(artifact.lpath).getnames()))
+
+        print('Tree extracted to %s' % self.stpath)
+
+        # After creating a bare-bone layout, create a tarball.
+        outname = "librdkafka-static-bundle-%s.tgz" % self.version
+        print('Writing to %s' % outname)
+        subprocess.check_call("(cd %s && tar cvzf ../%s .)" % \
+                              (self.stpath, outname),
+                              shell=True)
+
+        return outname
+
+
+    def verify (self, path):
+        """ Verify package """
+        expect = [
+            "./rdkafka.h",
+            "./LICENSES.txt",
+            "./librdkafka_glibc_linux.a",
+            "./librdkafka_glibc_linux.pc",
+            "./librdkafka_musl_linux.a",
+            "./librdkafka_musl_linux.pc",
+            "./librdkafka_darwin.a",
+            "./librdkafka_darwin.pc"]
+
+        missing = list()
+        with zfile.ZFile(path, 'r') as zf:
+            print('Verifying %s:' % path)
+
+            # Zipfiles may url-encode filenames, unquote them before matching.
+            pkgd = [unquote(x) for x in zf.getnames()]
             missing = [x for x in expect if x not in pkgd]
 
         if len(missing) > 0:
