@@ -193,6 +193,7 @@ struct rd_kafka_conf_s {
 	int     metadata_refresh_fast_interval_ms;
         int     metadata_refresh_sparse;
         int     metadata_max_age_ms;
+        int     metadata_propagation_max_ms;
 	int     debug;
 	int     broker_addr_ttl;
         int     broker_addr_family;
@@ -218,14 +219,13 @@ struct rd_kafka_conf_s {
 	char   *broker_version_fallback;
 	rd_kafka_secproto_t security_protocol;
 
-#if WITH_SSL
         struct {
+#if WITH_SSL
                 SSL_CTX *ctx;
+#endif
                 char *cipher_suites;
-#if OPENSSL_VERSION_NUMBER >= 0x1000200fL && !defined(LIBRESSL_VERSION_NUMBER)
                 char *curves_list;
                 char *sigalgs_list;
-#endif
                 char *key_location;
                 char *key_pem;
                 rd_kafka_cert_t *key;
@@ -249,7 +249,6 @@ struct rd_kafka_conf_s {
                                        char *errstr, size_t errstr_size,
                                        void *opaque);
         } ssl;
-#endif
 
         struct {
                 const struct rd_kafka_sasl_provider *provider;
@@ -271,20 +270,17 @@ struct rd_kafka_conf_s {
                 /* Hash size */
                 size_t         scram_H_size;
 #endif
-#if WITH_SASL_OAUTHBEARER
                 char *oauthbearer_config;
                 int   enable_oauthbearer_unsecure_jwt;
-
                 /* SASL/OAUTHBEARER token refresh event callback */
                 void (*oauthbearer_token_refresh_cb) (
                         rd_kafka_t *rk,
                         const char *oauthbearer_config,
                         void *opaque);
-#endif
         } sasl;
 
-#if WITH_PLUGINS
         char *plugin_paths;
+#if WITH_PLUGINS
         rd_list_t plugins;
 #endif
 
@@ -329,6 +325,7 @@ struct rd_kafka_conf_s {
 	int    fetch_error_backoff_ms;
         char  *group_id_str;
         char  *group_instance_id;
+        int    allow_auto_create_topics;
 
         rd_kafka_pattern_list_t *topic_blacklist;
         struct rd_kafka_topic_conf_s *topic_conf; /* Default topic config
@@ -388,6 +385,7 @@ struct rd_kafka_conf_s {
 	int    max_retries;
 	int    retry_backoff_ms;
 	int    batch_num_messages;
+        int    batch_size;
 	rd_kafka_compression_t compression_codec;
 	int    dr_err_only;
 
@@ -506,7 +504,7 @@ struct rd_kafka_conf_s {
 int rd_kafka_socket_cb_linux (int domain, int type, int protocol, void *opaque);
 int rd_kafka_socket_cb_generic (int domain, int type, int protocol,
                                 void *opaque);
-#ifndef _MSC_VER
+#ifndef _WIN32
 int rd_kafka_open_cb_linux (const char *pathname, int flags, mode_t mode,
                             void *opaque);
 #endif
@@ -553,6 +551,9 @@ struct rd_kafka_topic_conf_s {
 
 void rd_kafka_anyconf_destroy (int scope, void *conf);
 
+rd_bool_t rd_kafka_conf_is_modified (const rd_kafka_conf_t *conf,
+                                     const char *name);
+
 void rd_kafka_desensitize_str (char *str);
 
 void rd_kafka_conf_desensitize (rd_kafka_conf_t *conf);
