@@ -219,7 +219,7 @@ struct rd_kafka_op_s {
         rd_kafka_prio_t       rko_prio;   /**< In-queue priority.
                                            *   Higher value means higher prio*/
 
-	shptr_rd_kafka_toppar_t *rko_rktp;
+	rd_kafka_toppar_t    *rko_rktp;
 
         /*
 	 * Generic fields
@@ -289,6 +289,7 @@ struct rd_kafka_op_s {
 			int64_t offset;
 			char *errstr;
 			rd_kafka_msg_t rkm;
+                        rd_kafka_topic_t *rkt;
                         int fatal;  /**< This was a ERR__FATAL error that has
                                      *   been translated to the fatal error
                                      *   code. */
@@ -317,7 +318,7 @@ struct rd_kafka_op_s {
                 } metadata;
 
 		struct {
-			shptr_rd_kafka_itopic_t *s_rkt;
+			rd_kafka_topic_t *rkt;
 			rd_kafka_msgq_t msgq;
 			rd_kafka_msgq_t msgq2;
 			int do_purge2;
@@ -347,6 +348,7 @@ struct rd_kafka_op_s {
                         char fac[64];
                         int  level;
                         char *str;
+                        int  ctx;
                 } log;
 
                 struct {
@@ -527,14 +529,17 @@ int rd_kafka_op_reply (rd_kafka_op_t *rko, rd_kafka_resp_err_t err);
 			rd_kafka_log(rk, LOG_ERR, "ERROR", __VA_ARGS__); \
 			break;						\
 		}							\
-		rd_kafka_q_op_err((rk)->rk_rep, RD_KAFKA_OP_ERR, err, 0, \
-				  NULL, 0, __VA_ARGS__);		\
+		rd_kafka_q_op_err((rk)->rk_rep, err, __VA_ARGS__);      \
 	} while (0)
 
-void rd_kafka_q_op_err (rd_kafka_q_t *rkq, rd_kafka_op_type_t optype,
-                        rd_kafka_resp_err_t err, int32_t version,
-                        rd_kafka_toppar_t *rktp, int64_t offset,
-			const char *fmt, ...);
+void rd_kafka_q_op_err (rd_kafka_q_t *rkq, rd_kafka_resp_err_t err,
+                        const char *fmt, ...)
+        RD_FORMAT(printf, 3, 4);
+void rd_kafka_consumer_err (rd_kafka_q_t *rkq, int32_t broker_id,
+                            rd_kafka_resp_err_t err, int32_t version,
+                            const char *topic, rd_kafka_toppar_t *rktp,
+                            int64_t offset, const char *fmt, ...)
+        RD_FORMAT(printf, 8, 9);
 rd_kafka_op_t *rd_kafka_op_req0 (rd_kafka_q_t *destq,
                                  rd_kafka_q_t *recvq,
                                  rd_kafka_op_t *rko,
