@@ -503,6 +503,8 @@ static void rd_kafka_txn_partition_registered (rd_kafka_toppar_t *rktp) {
         TAILQ_REMOVE(&rk->rk_eos.txn_waitresp_rktps, rktp, rktp_txnlink);
         mtx_unlock(&rk->rk_eos.txn_pending_lock);
 
+        /* Not destroy()/keep():ing rktp since it just changes tailq. */
+
         TAILQ_INSERT_TAIL(&rk->rk_eos.txn_rktps, rktp, rktp_txnlink);
 }
 
@@ -553,7 +555,7 @@ static void rd_kafka_txn_handle_AddPartitionsToTxn (rd_kafka_t *rk,
 
         while (TopicCnt-- > 0) {
                 rd_kafkap_str_t Topic;
-                rd_kafka_itopic_t *rkt;
+                rd_kafka_topic_t *rkt;
                 int32_t PartCnt;
                 int p_actions = 0;
 
@@ -565,8 +567,7 @@ static void rd_kafka_txn_handle_AddPartitionsToTxn (rd_kafka_t *rk,
                         rd_kafka_topic_rdlock(rkt); /* for toppar_get() */
 
                 while (PartCnt-- > 0) {
-                        shptr_rd_kafka_toppar_t *s_rktp = NULL;
-                        rd_kafka_toppar_t *rktp;
+                        rd_kafka_toppar_t *rktp = NULL;
                         int32_t Partition;
                         int16_t ErrorCode;
 
@@ -574,11 +575,11 @@ static void rd_kafka_txn_handle_AddPartitionsToTxn (rd_kafka_t *rk,
                         rd_kafka_buf_read_i16(rkbuf, &ErrorCode);
 
                         if (rkt)
-                                s_rktp = rd_kafka_toppar_get(rkt,
-                                                             Partition,
-                                                             rd_false);
+                                rktp = rd_kafka_toppar_get(rkt,
+                                                           Partition,
+                                                           rd_false);
 
-                        if (!s_rktp) {
+                        if (!rktp) {
                                 rd_rkb_dbg(rkb, EOS|RD_KAFKA_DBG_PROTOCOL,
                                            "ADDPARTS",
                                            "Unknown partition \"%.*s\" "
@@ -589,8 +590,6 @@ static void rd_kafka_txn_handle_AddPartitionsToTxn (rd_kafka_t *rk,
                                 continue;
                         }
 
-                        rktp = rd_kafka_toppar_s2i(s_rktp);
-
                         switch (ErrorCode)
                         {
                         case RD_KAFKA_RESP_ERR_NO_ERROR:
@@ -614,6 +613,7 @@ static void rd_kafka_txn_handle_AddPartitionsToTxn (rd_kafka_t *rk,
                                 break;
 
                         case RD_KAFKA_RESP_ERR_TRANSACTIONAL_ID_AUTHORIZATION_FAILED:
+                        case RD_KAFKA_RESP_ERR_CLUSTER_AUTHORIZATION_FAILED:
                         case RD_KAFKA_RESP_ERR_INVALID_PRODUCER_ID_MAPPING:
                         case RD_KAFKA_RESP_ERR_INVALID_PRODUCER_EPOCH:
                         case RD_KAFKA_RESP_ERR_INVALID_TXN_STATE:
@@ -668,7 +668,7 @@ static void rd_kafka_txn_handle_AddPartitionsToTxn (rd_kafka_t *rk,
                                 okcnt++;
                         }
 
-                        rd_kafka_toppar_destroy(s_rktp);
+                        rd_kafka_toppar_destroy(rktp);
                 }
 
                 if (rkt) {
@@ -849,18 +849,22 @@ void rd_kafka_txn_schedule_register_partitions (rd_kafka_t *rk,
 
 
 /**
- * @brief Clears \p flag from all rktps in \p tqh
+ * @brief Clears \p flag from all rktps and destroys them, emptying
+ *        and reinitializing the \p tqh.
  */
 static void rd_kafka_txn_clear_partitions_flag (rd_kafka_toppar_tqhead_t *tqh,
                                                 int flag) {
-        rd_kafka_toppar_t *rktp;
+        rd_kafka_toppar_t *rktp, *tmp;
 
-        TAILQ_FOREACH(rktp, tqh, rktp_txnlink) {
+        TAILQ_FOREACH_SAFE(rktp, tqh, rktp_txnlink, tmp) {
                 rd_kafka_toppar_lock(rktp);
                 rd_dassert(rktp->rktp_flags & flag);
                 rktp->rktp_flags &= ~flag;
                 rd_kafka_toppar_unlock(rktp);
+                rd_kafka_toppar_destroy(rktp);
         }
+
+        TAILQ_INIT(tqh);
 }
 
 
@@ -874,8 +878,6 @@ static void rd_kafka_txn_clear_pending_partitions (rd_kafka_t *rk) {
                                            RD_KAFKA_TOPPAR_F_PEND_TXN);
         rd_kafka_txn_clear_partitions_flag(&rk->rk_eos.txn_waitresp_rktps,
                                            RD_KAFKA_TOPPAR_F_PEND_TXN);
-        TAILQ_INIT(&rk->rk_eos.txn_pending_rktps);
-        TAILQ_INIT(&rk->rk_eos.txn_waitresp_rktps);
 }
 
 /**
@@ -886,7 +888,6 @@ static void rd_kafka_txn_clear_pending_partitions (rd_kafka_t *rk) {
 static void rd_kafka_txn_clear_partitions (rd_kafka_t *rk) {
         rd_kafka_txn_clear_partitions_flag(&rk->rk_eos.txn_rktps,
                                            RD_KAFKA_TOPPAR_F_IN_TXN);
-        TAILQ_INIT(&rk->rk_eos.txn_rktps);
 }
 
 
@@ -905,13 +906,15 @@ rd_kafka_txn_curr_api_abort_timeout_cb (rd_kafka_timers_t *rkts, void *arg) {
         rd_kafka_txn_set_abortable_error(
                 rkts->rkts_rk,
                 RD_KAFKA_RESP_ERR__TIMED_OUT,
-                "Transactional operation timed out");
+                "Transactional API operation (%s) timed out",
+                rkq->rkq_rk->rk_eos.txn_curr_api.name);
 
         rd_kafka_txn_curr_api_reply_error(
                 rkq,
                 rd_kafka_error_new_txn_requires_abort(
                         RD_KAFKA_RESP_ERR__TIMED_OUT,
-                        "Transactional operation timed out"));
+                        "Transactional API operation (%s) timed out",
+                        rkq->rkq_rk->rk_eos.txn_curr_api.name));
 }
 
 /**
@@ -1304,6 +1307,7 @@ rd_kafka_txn_op_begin_transaction (rd_kafka_t *rk,
                 rd_kafka_txn_set_state(rk, RD_KAFKA_TXN_STATE_IN_TRANSACTION);
 
                 rk->rk_eos.txn_req_cnt = 0;
+                rd_atomic64_set(&rk->rk_eos.txn_dr_fails, 0);
                 rk->rk_eos.txn_err = RD_KAFKA_RESP_ERR_NO_ERROR;
                 RD_IF_FREE(rk->rk_eos.txn_errstr, rd_free);
                 rk->rk_eos.txn_errstr = NULL;
@@ -1457,6 +1461,7 @@ static void rd_kafka_txn_handle_TxnOffsetCommit (rd_kafka_t *rk,
                 break;
 
         case RD_KAFKA_RESP_ERR_TRANSACTIONAL_ID_AUTHORIZATION_FAILED:
+        case RD_KAFKA_RESP_ERR_CLUSTER_AUTHORIZATION_FAILED:
         case RD_KAFKA_RESP_ERR_INVALID_PRODUCER_ID_MAPPING:
         case RD_KAFKA_RESP_ERR_INVALID_PRODUCER_EPOCH:
         case RD_KAFKA_RESP_ERR_INVALID_TXN_STATE:
@@ -1667,6 +1672,7 @@ static void rd_kafka_txn_handle_AddOffsetsToTxn (rd_kafka_t *rk,
                 break;
 
         case RD_KAFKA_RESP_ERR_TRANSACTIONAL_ID_AUTHORIZATION_FAILED:
+        case RD_KAFKA_RESP_ERR_CLUSTER_AUTHORIZATION_FAILED:
         case RD_KAFKA_RESP_ERR_INVALID_PRODUCER_EPOCH:
         case RD_KAFKA_RESP_ERR_INVALID_TXN_STATE:
         case RD_KAFKA_RESP_ERR_UNSUPPORTED_FOR_MESSAGE_FORMAT:
@@ -1913,7 +1919,7 @@ static void rd_kafka_txn_handle_EndTxn (rd_kafka_t *rk,
         rd_kafka_q_t *rkq = opaque;
         int16_t ErrorCode;
         int actions = 0;
-        rd_bool_t is_commit = rd_false;
+        rd_bool_t is_commit = rd_false, may_retry = rd_false;
 
         if (err == RD_KAFKA_RESP_ERR__DESTROY) {
                 rd_kafka_q_destroy(rkq);
@@ -1933,28 +1939,42 @@ static void rd_kafka_txn_handle_EndTxn (rd_kafka_t *rk,
         err = rkbuf->rkbuf_err;
  err:
         rd_kafka_wrlock(rk);
-        if (rk->rk_eos.txn_state == RD_KAFKA_TXN_STATE_COMMITTING_TRANSACTION)
+        if (rk->rk_eos.txn_state == RD_KAFKA_TXN_STATE_COMMITTING_TRANSACTION) {
                 is_commit = rd_true;
-        else if (rk->rk_eos.txn_state ==
-                 RD_KAFKA_TXN_STATE_ABORTING_TRANSACTION)
+                may_retry = rd_true;
+        } else if (rk->rk_eos.txn_state ==
+                   RD_KAFKA_TXN_STATE_ABORTING_TRANSACTION) {
                 is_commit = rd_false;
-        else
+                may_retry = rd_true;
+        } else if (!err)
                 err = RD_KAFKA_RESP_ERR__OUTDATED;
+
+        if (!err) {
+                /* EndTxn successful: complete the transaction */
+                rd_kafka_txn_complete(rk);
+        }
+
+        rd_kafka_dbg(rk, EOS, "ENDTXN",
+                     "EndTxn failed due to %s in state %s (may_retry=%s)",
+                     rd_kafka_err2name(err),
+                     rd_kafka_txn_state2str(rk->rk_eos.txn_state),
+                     RD_STR_ToF(may_retry));
+
         rd_kafka_wrunlock(rk);
 
         switch (err)
         {
         case RD_KAFKA_RESP_ERR_NO_ERROR:
-                /* EndTxn successful: complete the transaction */
-                rd_kafka_txn_complete(rk);
                 break;
 
         case RD_KAFKA_RESP_ERR__DESTROY:
                 /* Producer is being terminated, ignore the response. */
+        case RD_KAFKA_RESP_ERR__TIMED_OUT:
+                /* Transaction API timeout has been hit
+                 * (this is our internal timer) */
         case RD_KAFKA_RESP_ERR__OUTDATED:
-                /* Set a non-actionable actions flag so that curr_api_reply()
-                 * is called below, without other side-effects. */
-                actions = RD_KAFKA_ERR_ACTION_SPECIAL;
+                /* Transactional state no longer relevant for this
+                 * outdated response. */
                 break;
 
         case RD_KAFKA_RESP_ERR_COORDINATOR_NOT_AVAILABLE:
@@ -1970,6 +1990,7 @@ static void rd_kafka_txn_handle_EndTxn (rd_kafka_t *rk,
 
         case RD_KAFKA_RESP_ERR_INVALID_PRODUCER_EPOCH:
         case RD_KAFKA_RESP_ERR_TRANSACTIONAL_ID_AUTHORIZATION_FAILED:
+        case RD_KAFKA_RESP_ERR_CLUSTER_AUTHORIZATION_FAILED:
         case RD_KAFKA_RESP_ERR_INVALID_TXN_STATE:
                 actions |= RD_KAFKA_ERR_ACTION_FATAL;
                 break;
@@ -1985,7 +2006,7 @@ static void rd_kafka_txn_handle_EndTxn (rd_kafka_t *rk,
                                              "Failed to end transaction: %s",
                                              rd_kafka_err2str(err));
 
-        } else if (actions & RD_KAFKA_ERR_ACTION_RETRY) {
+        } else if (may_retry && actions & RD_KAFKA_ERR_ACTION_RETRY) {
                 if (rd_kafka_buf_retry(rkb, request))
                         return;
                 actions |= RD_KAFKA_ERR_ACTION_PERMANENT;
@@ -2023,6 +2044,7 @@ rd_kafka_txn_op_commit_transaction (rd_kafka_t *rk,
         rd_kafka_resp_err_t err;
         char errstr[512];
         rd_kafka_pid_t pid;
+        int64_t dr_fails;
 
         if (rko->rko_err == RD_KAFKA_RESP_ERR__DESTROY)
                 return RD_KAFKA_OP_RES_HANDLED;
@@ -2043,6 +2065,18 @@ rd_kafka_txn_op_commit_transaction (rd_kafka_t *rk,
                 goto err;
         }
 
+        /* If any messages failed delivery the transaction must be aborted. */
+        dr_fails = rd_atomic64_get(&rk->rk_eos.txn_dr_fails);
+        if (unlikely(dr_fails > 0)) {
+                error = rd_kafka_error_new_txn_requires_abort(
+                        RD_KAFKA_RESP_ERR__INCONSISTENT,
+                        "%"PRId64" message(s) failed delivery "
+                        "(see individual delivery reports)",
+                        dr_fails);
+                goto err;
+        }
+
+
         err = rd_kafka_EndTxnRequest(rk->rk_eos.txn_coord,
                                      rk->rk_conf.eos.transactional_id,
                                      pid,
@@ -2065,6 +2099,14 @@ rd_kafka_txn_op_commit_transaction (rd_kafka_t *rk,
  err:
         rd_kafka_wrunlock(rk);
 
+        /* If the returned error is an abortable error
+         * also set the current transaction state accordingly. */
+        if (rd_kafka_error_txn_requires_abort(error))
+                rd_kafka_txn_set_abortable_error(
+                        rk,
+                        rd_kafka_error_code(error),
+                        "%s", rd_kafka_error_string(error));
+
         rd_kafka_txn_curr_api_reply_error(rd_kafka_q_keep(rko->rko_replyq.q),
                                           error);
 
@@ -2485,6 +2527,7 @@ rd_kafka_txn_handle_FindCoordinator (rd_kafka_t *rk,
                 return;
 
         case RD_KAFKA_RESP_ERR_TRANSACTIONAL_ID_AUTHORIZATION_FAILED:
+        case RD_KAFKA_RESP_ERR_CLUSTER_AUTHORIZATION_FAILED:
                 rd_kafka_wrlock(rk);
                 rd_kafka_txn_set_fatal_error(
                         rkb->rkb_rk, RD_DONT_LOCK, err,
@@ -2750,5 +2793,6 @@ void rd_kafka_txns_init (rd_kafka_t *rk) {
         rd_kafka_broker_persistent_connection_add(
                 rk->rk_eos.txn_coord,
                 &rk->rk_eos.txn_coord->rkb_persistconn.coord);
-}
 
+        rd_atomic64_init(&rk->rk_eos.txn_dr_fails, 0);
+}
