@@ -75,4 +75,8 @@ void *rd_strtup_list_copy (const void *elem, void *opaque);
 char *rd_flags2str (char *dst, size_t size,
                     const char **desc, int flags);
 
+
+/** @returns "true" if EXPR is true, else "false" */
+#define RD_STR_ToF(EXPR) ((EXPR) ? "true" : "false")
+
 #endif /* _RDSTRING_H_ */
