@@ -68,6 +68,8 @@ void do_produce_batch (rd_kafka_t *rk, const char *topic, uint64_t testid,
         ret = rd_kafka_produce_batch(rkt, partition, RD_KAFKA_MSG_F_COPY,
                                      messages, cnt);
 
+        rd_kafka_topic_destroy(rkt);
+
         TEST_ASSERT(ret == cnt,
                     "Failed to batch-produce: %d/%d messages produced",
                     ret, cnt);
@@ -92,7 +94,7 @@ void do_produce_batch (rd_kafka_t *rk, const char *topic, uint64_t testid,
  *        (only consumed output for verification).
  *        e.g., no consumer offsets to commit with transaction.
  */
-static void do_test_basic_producer_txn (void) {
+static void do_test_basic_producer_txn (rd_bool_t enable_compression) {
         const char *topic = test_mk_topic_name("0103_transactions", 1);
         const int partition_cnt = 4;
 #define _TXNCNT 6
@@ -129,6 +131,8 @@ static void do_test_basic_producer_txn (void) {
         p_conf = rd_kafka_conf_dup(conf);
         rd_kafka_conf_set_dr_msg_cb(p_conf, test_dr_msg_cb);
         test_conf_set(p_conf, "transactional.id", topic);
+        if (enable_compression)
+                test_conf_set(p_conf, "compression.type", "lz4");
         p = test_create_handle(RD_KAFKA_PRODUCER, p_conf);
 
         // FIXME: add testing were the txn id is reused (and thus fails)
@@ -377,7 +381,7 @@ void do_test_consumer_producer_txn (void) {
         test_msgver_init(&expect_mv, testid);
 
         for (txn = 0 ; txn < txncnt ; txn++) {
-                int msgcnt = 10 * (1 + (txn % 3));
+                int msgcnt2 = 10 * (1 + (txn % 3));
                 rd_kafka_message_t *msgs[_MSGCNT];
                 int i;
                 rd_bool_t do_abort = !(txn % 3);
@@ -385,19 +389,19 @@ void do_test_consumer_producer_txn (void) {
                 rd_kafka_topic_partition_list_t *offsets;
                 rd_kafka_resp_err_t err;
                 rd_kafka_consumer_group_metadata_t *c1_cgmetadata;
-                int remains = msgcnt;
+                int remains = msgcnt2;
 
                 TEST_SAY(_C_BLU "Begin transaction #%d/%d "
                          "(msgcnt=%d, do_abort=%s, recreate_consumer=%s)\n",
-                         txn, txncnt, msgcnt,
+                         txn, txncnt, msgcnt2,
                          do_abort ? "true":"false",
                          recreate_consumer ? "true":"false");
 
-                consume_messages(c1, msgs, msgcnt);
+                consume_messages(c1, msgs, msgcnt2);
 
                 TEST_CALL_ERROR__(rd_kafka_begin_transaction(p2));
 
-                for (i = 0 ; i < msgcnt ; i++) {
+                for (i = 0 ; i < msgcnt2 ; i++) {
                         rd_kafka_message_t *msg = msgs[i];
 
                         if (!do_abort) {
@@ -429,7 +433,7 @@ void do_test_consumer_producer_txn (void) {
                         rd_kafka_poll(p2, 0);
                 }
 
-                destroy_messages(msgs, msgcnt);
+                destroy_messages(msgs, msgcnt2);
 
                 err = rd_kafka_assignment(c1, &offsets);
                 TEST_ASSERT(!err, "failed to get consumer assignment: %s",
@@ -537,6 +541,7 @@ static void do_test_misuse_txn (void) {
                     error ? rd_kafka_error_string(error) : "");
         TEST_ASSERT(rd_kafka_error_is_fatal(error),
                     "Expected error to have is_fatal() set");
+        rd_kafka_error_destroy(error);
         /* Check that a fatal error is raised */
         fatal_err = rd_kafka_fatal_error(p, errstr, sizeof(errstr));
         TEST_ASSERT(fatal_err == RD_KAFKA_RESP_ERR_INVALID_TRANSACTION_TIMEOUT,
@@ -565,6 +570,7 @@ static void do_test_misuse_txn (void) {
         TEST_ASSERT(rd_kafka_error_code(error) == RD_KAFKA_RESP_ERR__STATE,
                     "Expected ERR__STATE error, not %s",
                     rd_kafka_error_name(error));
+        rd_kafka_error_destroy(error);
 
         TEST_CALL_ERROR__(rd_kafka_begin_transaction(p));
 
@@ -573,6 +579,7 @@ static void do_test_misuse_txn (void) {
         TEST_ASSERT(rd_kafka_error_code(error) == RD_KAFKA_RESP_ERR__STATE,
                     "Expected ERR__STATE error, not %s",
                     rd_kafka_error_name(error));
+        rd_kafka_error_destroy(error);
 
         rd_kafka_destroy(p);
 
@@ -596,6 +603,7 @@ static void do_test_misuse_txn (void) {
                     rd_kafka_error_string(error));
         TEST_ASSERT(rd_kafka_error_is_retriable(error),
                     "Expected error to be retriable");
+        rd_kafka_error_destroy(error);
 
         TEST_CALL_ERROR__(rd_kafka_init_transactions(p, 30*1000));
 
@@ -765,7 +773,8 @@ static void do_test_fenced_txn (rd_bool_t produce_after_fence) {
 int main_0103_transactions (int argc, char **argv) {
 
         do_test_misuse_txn();
-        do_test_basic_producer_txn();
+        do_test_basic_producer_txn(rd_false /* without compression */);
+        do_test_basic_producer_txn(rd_true /* with compression */);
         do_test_consumer_producer_txn();
         do_test_fenced_txn(rd_false /* no produce after fencing */);
         do_test_fenced_txn(rd_true /* produce after fencing */);
